"""Add logs column on kernel table

Revision ID: 51dddd79aa21
Revises: 3bb80d1887d6
Create Date: 2020-02-11 14:45:55.496745

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "51dddd79aa21"
down_revision = "3bb80d1887d6"
branch_labels = None
depends_on = None


def upgrade():
    op.add_column("kernels", sa.Column("container_log", sa.LargeBinary(), nullable=True))


def downgrade():
    op.drop_column("kernels", "container_log")



"""add-missing-vfolder-indexes

Revision ID: 41f332243bf9
Revises: 7ff52ff68bfc
Create Date: 2024-02-28 17:27:40.387122

"""

from alembic import op

# revision identifiers, used by Alembic.
revision = "41f332243bf9"
down_revision = "7ff52ff68bfc"
branch_labels = None
depends_on = None


def upgrade():
    op.create_index(op.f("ix_vfolders_host"), "vfolders", ["host"], unique=False)
    op.create_index(
        op.f("ix_vfolders_ownership_type"), "vfolders", ["ownership_type"], unique=False
    )
    op.create_index(op.f("ix_vfolders_status"), "vfolders", ["status"], unique=False)
    op.create_index(op.f("ix_vfolders_usage_mode"), "vfolders", ["usage_mode"], unique=False)


def downgrade():
    op.drop_index(op.f("ix_vfolders_usage_mode"), table_name="vfolders")
    op.drop_index(op.f("ix_vfolders_status"), table_name="vfolders")
    op.drop_index(op.f("ix_vfolders_ownership_type"), table_name="vfolders")
    op.drop_index(op.f("ix_vfolders_host"), table_name="vfolders")



"""more_status

Revision ID: 0f3bc98edaa0
Revises: 7ea324d0535b
Create Date: 2017-08-11 13:12:55.236519

"""

import sqlalchemy as sa
from alembic import op
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "0f3bc98edaa0"
down_revision = "7ea324d0535b"
branch_labels = None
depends_on = None

agentstatus = postgresql.ENUM(
    "ALIVE",
    "LOST",
    "RESTARTING",
    "TERMINATED",
    name="agentstatus",
)

kernelstatus_choices = (
    "PREPARING",
    "BUILDING",
    "RUNNING",
    "RESTARTING",
    "RESIZING",
    "SUSPENDED",
    "TERMINATING",
    "TERMINATED",
    "ERROR",
)
kernelstatus = postgresql.ENUM(*kernelstatus_choices, name="kernelstatus")


def upgrade():
    agentstatus.create(op.get_bind())
    kernelstatus.create(op.get_bind())
    op.add_column("agents", sa.Column("lost_at", sa.DateTime(timezone=True), nullable=True))
    op.add_column(
        "agents",
        sa.Column(
            "status",
            sa.Enum("ALIVE", "LOST", "RESTARTING", "TERMINATED", name="agentstatus"),
            nullable=False,
        ),
    )
    op.create_index(op.f("ix_agents_status"), "agents", ["status"], unique=False)
    op.add_column("kernels", sa.Column("agent_addr", sa.String(length=128), nullable=False))
    op.add_column("kernels", sa.Column("cpu_slot", sa.Integer(), nullable=False))
    op.add_column("kernels", sa.Column("gpu_slot", sa.Integer(), nullable=False))
    op.add_column("kernels", sa.Column("mem_slot", sa.Integer(), nullable=False))
    op.add_column("kernels", sa.Column("repl_in_port", sa.Integer(), nullable=False))
    op.add_column("kernels", sa.Column("repl_out_port", sa.Integer(), nullable=False))
    op.add_column("kernels", sa.Column("stdin_port", sa.Integer(), nullable=False))
    op.add_column("kernels", sa.Column("stdout_port", sa.Integer(), nullable=False))
    op.drop_column("kernels", "allocated_cores")
    op.add_column("kernels", sa.Column("cpu_set", sa.ARRAY(sa.Integer), nullable=True))
    op.add_column("kernels", sa.Column("gpu_set", sa.ARRAY(sa.Integer), nullable=True))
    op.alter_column(
        "kernels",
        column_name="status",
        type_=sa.Enum(*kernelstatus_choices, name="kernelstatus"),
        postgresql_using="status::kernelstatus",
    )


def downgrade():
    op.drop_column("kernels", "stdout_port")
    op.drop_column("kernels", "stdin_port")
    op.drop_column("kernels", "repl_out_port")
    op.drop_column("kernels", "repl_in_port")
    op.drop_column("kernels", "mem_slot")
    op.drop_column("kernels", "gpu_slot")
    op.drop_column("kernels", "cpu_slot")
    op.drop_column("kernels", "agent_addr")
    op.drop_index(op.f("ix_agents_status"), table_name="agents")
    op.drop_column("agents", "status")
    op.drop_column("agents", "lost_at")
    op.alter_column(
        "kernels", column_name="status", type_=sa.String(length=64), server_default=None
    )
    op.add_column("kernels", sa.Column("allocated_cores", sa.ARRAY(sa.Integer), nullable=True))
    op.drop_column("kernels", "cpu_set")
    op.drop_column("kernels", "gpu_set")
    agentstatus.drop(op.get_bind())
    kernelstatus.drop(op.get_bind())



"""add_ssh_keypair_into_keypair

Revision ID: 0262e50e90e0
Revises: 4b7b650bc30e
Create Date: 2019-12-12 07:19:48.052928

"""

import sqlalchemy as sa
from alembic import op
from cryptography.hazmat.backends import default_backend as crypto_default_backend
from cryptography.hazmat.primitives import serialization as crypto_serialization
from cryptography.hazmat.primitives.asymmetric import rsa

from ai.backend.manager.models.base import convention

# revision identifiers, used by Alembic.
revision = "0262e50e90e0"
down_revision = "4b7b650bc30e"
branch_labels = None
depends_on = None


def generate_ssh_keypair():
    key = rsa.generate_private_key(
        backend=crypto_default_backend(), public_exponent=65537, key_size=2048
    )
    private_key = key.private_bytes(
        crypto_serialization.Encoding.PEM,
        crypto_serialization.PrivateFormat.TraditionalOpenSSL,
        crypto_serialization.NoEncryption(),
    ).decode("utf-8")
    public_key = (
        key.public_key()
        .public_bytes(
            crypto_serialization.Encoding.OpenSSH, crypto_serialization.PublicFormat.OpenSSH
        )
        .decode("utf-8")
    )
    return (public_key, private_key)


def upgrade():
    op.add_column("keypairs", sa.Column("ssh_public_key", sa.String(length=750), nullable=True))
    op.add_column("keypairs", sa.Column("ssh_private_key", sa.String(length=2000), nullable=True))

    # partial table to be preserved and referred
    metadata = sa.MetaData(naming_convention=convention)
    keypairs = sa.Table(
        "keypairs",
        metadata,
        sa.Column("access_key", sa.String(length=20), primary_key=True),
        sa.Column("ssh_public_key", sa.String(length=750), nullable=True),
        sa.Column("ssh_private_key", sa.String(length=2000), nullable=True),
    )

    # Fill in SSH keypairs in every keypairs.
    conn = op.get_bind()
    query = sa.select([keypairs.c.access_key]).select_from(keypairs)
    rows = conn.execute(query).fetchall()
    for row in rows:
        pubkey, privkey = generate_ssh_keypair()
        query = (
            sa.update(keypairs)
            .values(ssh_public_key=pubkey, ssh_private_key=privkey)
            .where(keypairs.c.access_key == row.access_key)
        )
        conn.execute(query)


def downgrade():
    op.drop_column("keypairs", "ssh_public_key")
    op.drop_column("keypairs", "ssh_private_key")



"""add model service token log

Revision ID: 02535458c0b3
Revises: ae7d4cd92aa7
Create Date: 2023-08-25 03:34:01.750965

"""

from enum import Enum

import sqlalchemy as sa
from alembic import op
from sqlalchemy.dialects import postgresql

from ai.backend.manager.models.base import GUID, EnumValueType, ForeignKeyIDColumn

# revision identifiers, used by Alembic.
revision = "02535458c0b3"
down_revision = "ae7d4cd92aa7"
branch_labels = None
depends_on = None


class EndpointLifecycle(Enum):
    CREATED = "created"
    DESTROYING = "destroying"
    DESTROYED = "destroyed"


endpointlifecycle_choices = [v.value for v in EndpointLifecycle]
endpointlifecycle = postgresql.ENUM(*endpointlifecycle_choices, name="endpointlifecycle")


def upgrade():
    endpointlifecycle.create(op.get_bind())
    op.create_table(
        "endpoint_tokens",
        sa.Column("token", sa.VARCHAR(1024), primary_key=True),
        sa.Column("domain", sa.String(length=64), nullable=False),
        sa.Column("project", GUID(), nullable=False),
        ForeignKeyIDColumn("endpoint", "endpoints.id"),
        ForeignKeyIDColumn("session_owner", "users.uuid"),
        sa.Column(
            "created_at", sa.DateTime(timezone=True), server_default=sa.text("now()"), nullable=True
        ),
    )
    op.add_column(
        "endpoints",
        sa.Column(
            "created_at", sa.DateTime(timezone=True), server_default=sa.text("now()"), nullable=True
        ),
    )
    op.add_column(
        "endpoints",
        sa.Column("destroyed_at", sa.DateTime(timezone=True), nullable=True),
    )
    op.add_column(
        "endpoints",
        sa.Column(
            "lifecycle_stage",
            EnumValueType(EndpointLifecycle),
            nullable=False,
            default=EndpointLifecycle.CREATED,
            server_default="created",
        ),
    )
    op.add_column(
        "routings",
        sa.Column(
            "created_at", sa.DateTime(timezone=True), server_default=sa.text("now()"), nullable=True
        ),
    )


def downgrade():
    op.drop_column("endpoints", "created_at")
    op.drop_column("endpoints", "destroyed_at")
    op.drop_column("endpoints", "lifecycle_stage")
    op.drop_column("routings", "created_at")
    op.drop_table("endpoint_tokens")
